;**************************************************************************
; Source code for Direct Digital Synthesis VFO controller
;
; Filename:     9851_19.ASM
;
; This code is available as is.
; 
; Description:
; The program main loop, (Check_Encoder) polls a rotary shaft encoder.
; If the input from the shaft encoder has changed, then direction (up/down)
; is determined.  The program then computes the corresponding
; frequency digits to be displayed on the Liquid Crystal Display. 
; This data is transfered to the LCD using a 4 bit interface. 
; This LCD Display routine is configured to show only KHZ digits and 
; display readouts in MHz.  A constant 1 KHz tuning step size is used.
; A new DDS control word is then computed and transfered
; serially to the AD9851 DDS chip.
; The program then returns to the shaft encoder polling loop.
;
; This version of 9851_* is designed to drive a receiver with 10.7 MHz 
; IF offset and an optrex LCD.  
; 
; This code is copyright 1999 Mycrocomm Labs.  
;
;**************************************************************************
	   list  p=16f84
	   radix hex
;
Indirect   equ  0
Status     equ  3
FSR        equ  4
PortA      equ  5
PortB      equ  6

;Assign names to IO pins
LCD_VDD    equ  3              ;(PortA)Power to LCD module. (not used)
strb_5821  equ  3              ;(PortA)Strobe line to UCN5821A  
AD9850_fqu equ  1              ;(PortA)Update pin on AD9850
LCD_rs     equ  2              ;(PortA)0=instruction, 1=data
LCD_rw     equ  3              ;(PortA)0=write, 1=read
LCD_e      equ  4              ;(PortA)0=disable, 1=enable
AD9850_wc  equ  1              ;(PortB)AD9850 write clock
AD9850_dat equ  0              ;(PortB)AD9850 serial data input
;
;Allocate variables in general purpose register space
LCD_Dat    equ  0x0C           ;current Display data    
AD9850     equ  0x13           ;control word for DDS chip 
OP1        equ  0x18           ;Work Space for                      
OP2        equ  0x1C           ;  the multiply routine
temp1      equ  0x1D           ;Temp storage         
temp2      equ  0x1E           ;Temp storage          
temp3      equ  0x1F           ;Temp storage           
old        equ  0x20           ;old encoder input data
new        equ  0x21           ;new encoder input data
count1     equ  0x22           ;used for timing the shaft encoder 
count2     equ  0x23           ; 
new_sb     equ  0x24            ;latest state of U/L switch
old_sb     equ  0x25            ;old state of U/L switch    
Old_band   equ  0x26            ;Checksum from last band update                    
;
;Status Bits
C          equ  0              ;Carry
DC         equ  1              ;Digit (Half) Carry
Z          equ  2              ;Zero
;      
;
     org     0
     goto    Start             ;
;
     org     5
;**************************************************************************
;BusyCheck Subroutine
;
;LCD read/write operations are slooooow, this subroutine
;polls the LCD busy flag to determine if previous operations are completed.
;Note side effect that the LCD_rw and LCD_rs are left in write data mode. 
;
BusyCheck 
     movlw   0xFF              ;Tristate the 4 data pins
     tris    PortB             ;All Inputs
     bsf     PortA,LCD_rw      ;Raise the r/w bit to read the LCD
Busy
     bsf     PortA,LCD_e       ;
     nop
     movf    PortB,w           ;Get first nibble of input data
     movwf   temp1             ;  and save it 
     bcf     PortA,LCD_e       ;
     nop                       ;
     bsf     PortA,LCD_e       ;Get next nibble to keep LCD happy
     nop
     bcf     PortA,LCD_e       ;
     btfsc   temp1,3           ;Check for busy flag 
     goto    Busy              ;It is busy
     bcf     PortA,LCD_rw      ;Leave LCD in Write Data mode
     bsf     PortA,LCD_rs      ; 
     return
;   
;
;**************************************************************************
;Send Character to LCD Subroutine
;
; This routine optimized for Optrex 1x16 LCD display
;
;The character to be sent must have been placed in "temp2" prior to calling
;the subroutine. LCD_rw and LCD_rs must be set up prior to entry.
;
SendChar
     movlw   0xF0              ;
     tris    PortB             ;Enable port B
     movf    temp2,w           ;Save temp2 
     movwf   temp1             ;  in temp1
     call    Flipbit           ;Non-Destructive send to portb
     bsf     PortA,LCD_e       ;Transfer first nibble
     nop
     nop
     nop
     bcf       PortA,LCD_e       ;

     swapf     temp2,f
     call      Flipbit
     bsf       PortA,LCD_e       ;Transfer second nibble
     nop
     nop
     nop
     bcf       PortA,LCD_e       ;
     return
;
;**************************************************************************
;BCD increment
;
;This routine does a binary coded decimal increment to the values in LCD_Dat.
;On entry the FSR register must point to the LCD_Dat digit to be incremented. 
;If the result is over 60MHz the value is reset back to 60MHz.
;
;
BCDInc
     movf    Indirect,w        ; 
     movwf   temp2             ;
     movlw   0x01              ;
     addwf   temp2,f           ;    
     movf    temp2,w           ;
     movwf   temp1             ;     
     movlw   0x06              ;
     addwf   temp1,f           ;
     btfsc   Status, DC        ;See if  Digit Carry is clear
     goto    BCDIncCarry       ;Digit Carry is set, so jump
     movlw   0x0F              ;
     andwf   temp2,f           ;
     movf    temp2,w           ;
     movwf   Indirect          ;
     movlw   0x03             ; Set for F(max) of 60 MHz     
     subwf   LCD_Dat+0,w       ;Test for max frequency
     btfsc   Status, C         ;See if Carry is clear
     goto    SetMax            ;Carry is set, so jump
     return
BCDIncCarry
     clrf    temp2             ;
     movf    temp2,w           ;
     movwf   Indirect          ; 
     decf    FSR,f             ; 
     goto    BCDInc            ; 
SetMax
     movlw   0x12              ;Set the display to 30,000.00
     movwf   FSR               ;
SetMax1   
     movlw   0x00              ;
     movwf   Indirect          ;
     decf    FSR,f             ;  
     movlw   0xF3              ;
     addwf   FSR,w             ;
     btfsc   Status, C         ;See if Carry is clear
     goto    SetMax1           ;Carry is set, so jump
     movlw   0x03              ;Set back to 30 MHz
     movwf   Indirect          ;
     return                    ;
;**************************************************************************
;BCD decrement
;
;This routine does a binary coded decimal decrement to the values in LCD_Dat.
;On entry the FSR register must point to the LCD_Dat digit to be decremented,
;(one of 10's, 100's, 1K, 10K or 100K digit) 
;If the result is under 1KHz the value is reset back to 1KHz.
;
;
BCDDec
     movf    Indirect,w        ; 
     movwf   temp2             ;
     movlw   0x01              ;
     subwf   temp2,f           ;
     btfss   Status, DC        ;See if Digit Carry is set
     goto    BCDDecCarry       ;Digit Carry is clear, so jump
     movf    temp2,w           ;
     movwf   Indirect          ;
     movlw   0x09              ;check to see if 10M = 9
     subwf   LCD_Dat+0,w        ;
     btfsc   Status,Z          ;is result = 0?
     goto    SetMin            ; Yes.  Set dds for 1KHz         
     movlw   0xFF              ;
     addwf   LCD_Dat+0,w       ;  
     btfsc   Status, C         ;See if Carry is clear
     goto    BCDDecExit        ;Carry is set, so jump to exit
     movlw   0xFF              ;  
     addwf   LCD_Dat+1,w       ; 
     btfsc   Status, C         ;See if Carry is clear
     goto    BCDDecExit        ;Carry is set, so jump to exit
     movlw   0xFF              ;
     addwf   LCD_Dat+2,w       ; 
     btfsc   Status, C         ;See if Carry is clear
     goto    BCDDecExit        ;Carry is set, so jump to exit
     addwf   LCD_Dat+3,w       ; 
     btfsc   Status, C         ;See if Carry is clear
     goto    BCDDecExit        ;Carry is set, so jump to exit
     addwf   LCD_Dat+4,w       ; 
     btfsc   Status, C         ;See if Carry is clear
     goto    BCDDecExit        ;Carry is set, so jump to exit
     goto    SetMin            ;
BCDDecExit
     return
BCDDecCarry
     movlw   0x09              ;
     movwf   temp2             ;
     movf    temp2,w           ; 
     movwf   Indirect          ; 
     decf    FSR,f             ;
     goto    BCDDec            ;
SetMin
     movlw   0x12              ;
     movwf   FSR               ;
SetMin1
     movlw   0x00              ;
     movwf   Indirect          ;
     decf    FSR,f             ;
     movlw   0xEF              ; 
     addwf   FSR,w             ;
     btfsc   Status, C         ;See if Carry is clear
     goto    SetMin1           ;Carry is set, so jump 
     movlw   0x01              ;
     movwf   Indirect          ;Put '1' in KHz pos'n
     decf    FSR,f             ;
     movlw   0x00              ;
     movwf   Indirect          ;Put '0' in 10KHz pos'n
     decf    FSR,f             ;
     movlw   0x00              ;
     movwf   Indirect          ;Put '0' in 100KHz pos'n
     decf    FSR,f             ;
     movlw   0x00              ;
     movwf   Indirect          ;Put '0' in 1MHz pos'n
     decf    FSR,f             ;
     movlw   0x00              ;
     movwf   Indirect          ;Put '0' in 10MHz pos'n
     return                    ;
;
;**************************************************************************
; Delay 24ms subroutine
;
;
Wait24ms
     movlw   0x28              ; use 40 for 24ms wait if clock is 4Mhz
     movwf   temp1             ;
Wait24Outer
     movlw   0xC8              ; use 200 for 24 ms wait
     movwf   temp2             ;
Wait24Inner
     decfsz  temp2,f           ;
     goto    Wait24Inner       ;
     decfsz  temp1,f           ;
     goto    Wait24Outer       ;
     return
;
;
;**************************************************************************
; Delay 60ms subroutine
;
Wait60ms
     movlw   0x64              ; use 100 for 60ms wait if clock is 4Mhz
     movwf   temp1             ;
Wait60Outer
     movlw   0xC8              ; use 200 for 60ms wait
     movwf   temp2             ;
Wait60Inner
     decfsz  temp2,f           ;
     goto    Wait60Inner       ;
     decfsz  temp1,f           ;
     goto    Wait60Outer       ;
     return                    ;

;
;**************************************************************************
; 32 bit add
;
;The value in registers OP1 is added to the current 32 bit value in AD9850

;
Add32
     movf    OP1+0,w           ;
     addwf   AD9850+0,f        ;
     btfss   Status, C         ;See if Carry is set
     goto    Add1              ;Carry is clear, so jump
     incfsz  AD9850+1,f        ;
     goto    Add1              ; 
     incfsz  AD9850+2,f        ;
     goto    Add1              ;
     incf    AD9850+3,f        ;
Add1
     movf    OP1+1,w           ;
     addwf   AD9850+1,f        ;
     btfss   Status, C         ;See if Carry is set
     goto    Add2              ;Carry is clear, so jump
     incfsz  AD9850+2,f        ;
     goto    Add2              ;
     incf    AD9850+3,f        ;
Add2
     movf    OP1+2,w           ;
     addwf   AD9850+2,f        ;
     btfss   Status, C         ;See if Carry is set
     goto    Add3              ;Carry is clear, so jump
     incf    AD9850+3,f        ;
Add3    
     movf    OP1+3,w           ; 
     addwf   AD9850+3,f        ; 
     return                    ;
;
;
;**************************************************************************
;Write Data
;
;This subroutine does two things:
; (1) it sends the contents LCD_dat to the LCD,
;     (LCD_Dat must contain BCD digits)
; (2) the contents of AD9851 (40 bits) are serial shifted
;     into the AD9851 module
;
WriteData
     movlw   0x81              ; LCD position for first digit
     movwf   temp2             ;
     call    BusyCheck         ;
     bcf     PortA,LCD_rs      ; Set LCD for instruction write
     call    SendChar          ;
     movf    LCD_Dat+0,w       ;  
     movwf   temp2             ;
     movlw   0x20              ;Send a space if first digit is zero
     iorwf   temp2,f           ;
     movlw   0x20              ;Set up for space check
     subwf   temp2,w           ;Subtract to compare
     btfsc   Status, Z         ;See if Zero is clear (not a space)  
     goto    Wd2               ;Zero is set, so go send the space
     movlw   0x10              ;Zero is clear, so convert BCD 
     iorwf   temp2,f           ;  to ASCII
Wd2
     call    BusyCheck         ;
     call    SendChar          ;Send 10Mhz digit
     movf    LCD_Dat+1,w       ;
     movwf   temp2             ;
     movlw   0x30              ;
     iorwf   temp2,f           ;
     call    BusyCheck         ;
     call    SendChar          ;Send 1Mhz digit
     movlw   0x2E              ;
     movwf   temp2             ;
     call    BusyCheck         ;  
     call    SendChar          ;Send a Period
     movf    LCD_Dat+2,w       ; 
     movwf   temp2             ;
     movlw   0x30              ;
     iorwf   temp2,f
     call    BusyCheck         ;
     call    SendChar          ;Send a 100KHz digit
     movf    LCD_Dat+3,w
     movwf   temp2
     movlw   0x30
     iorwf   temp2,f
     call    BusyCheck         ;Send 10 KHz digit
     call    SendChar
;
     movf    LCD_Dat+4,w       ;
     movwf   temp2
     movlw   0x30
     iorwf   temp2,f
     call    BusyCheck
     call    SendChar          ; Send 1Khz digit
     movlw   0x20              ; Space
     movwf   temp2             ;
     call    BusyCheck         ;  
     call    SendChar          ;
     movlw   0x2E
     movwf   temp2
     ;call    BusyCheck
     ;call    SendChar          ; Send a period
     movlw   0xC0               ;Point to gap in Optrex char. Addr.
     movwf   temp2
     call    BusyCheck
     bcf     PortA,LCD_rs      ; Set LCD to Instruction Mode
     call    SendChar          ; 
     ;movf    LCD_Dat+5,w       ; 
     ;movwf   temp2
     ;movlw   0x30
     ;iorwf   temp2,f
     ;call    BusyCheck
     ;call    SendChar          ; Send 100Hz digit
     ;movf    LCD_Dat+6,w       ; 
     ;movwf   temp2
     ;movlw   0x30
     ;iorwf   temp2,f
     ;call    BusyCheck         ;
     ;call    SendChar          ; Send 10Hz digit
     ;movlw   0x00             ; Force Hz digit to 0
     ;movwf   temp2
     ;movlw   0x30
     ;iorwf   temp2,f
     ;call    BusyCheck         ;
     ;call    SendChar          ; Send 1Hz digit
     movlw   0x20              ; Space
     movwf   temp2             ;
     call    BusyCheck         ;  
     call    SendChar          ;
;       insert routine here to send "KHz" & "mode"
     movlw   0x4D              ; "M"
     movwf   temp2             ;
     call    BusyCheck         ;  
     call    SendChar          ;
     movlw   0x48              ; "H"
     movwf   temp2             ;
     call    BusyCheck         ;  
     call    SendChar          ;
     movlw   0x7A              ; "z"
     movwf   temp2             ;
     call    BusyCheck         ;  
     call    SendChar          ;
     movlw   0x20              ; Space
     movwf   temp2             ;
     call    BusyCheck         ;  
     call    SendChar          ;
     movlw   0xC8              ;Point LCD at off-display address
     movwf   temp2
     call    BusyCheck         ;
     bcf     PortA,LCD_rs      ;Set LCD for instruction write
     call    SendChar          ;
WriteAD9850
		
     clrf    PortB             ; Clear PortB
     movlw   0x05              ; Send 5 bytes
     movwf   temp3             ;   to AD9850
     movlw   0xF0              ; 
     tris    PortB             ; Enable PortB
     movlw   0x13              ; Point at AD9850+0
     movwf   FSR
NextByte
     movf    Indirect,w        ; 
     movwf   temp1 
     movlw   0x08              ; set bit counter to 8
     movwf   temp2
NextBit
     rrf     temp1,f
     btfss   Status, C         ;See if carry is set
     goto    Send0             ;Carry is clear, so jump
     bsf     PortB,AD9850_dat
     bsf     PortB,AD9850_wc   ; toggle write clock
     bcf     PortB,AD9850_wc   ; (repeated 40 times)
     goto    Break
Send0
     bcf     PortB,AD9850_dat
     bsf     PortB,AD9850_wc
     bcf     PortB,AD9850_wc
Break
     decfsz  temp2,f
     goto    NextBit
     incf    FSR,f
     decfsz  temp3,f
     goto    NextByte
     bsf     PortA,AD9850_fqu  ; send load signal to AD9850
     nop
     bcf     PortA,AD9850_fqu  ; Clear after last data dbit is written
     return
;
;***************************************************************************  
;7 Digit BCD by 32 bit binary mulitply
;
;The 7 BCD digits stored in LCD_dat are each multiplied by a precalculated digit
;weight, (Digit weigths assume a 180 MHz input clock). The sum of all 7
;digits times their weight is stored in the AD9851 registers.
;
Mult
     clrf    temp1  
     clrf    AD9850+0
     clrf    AD9850+1
     clrf    AD9850+2
     clrf    AD9850+3
     movlw   0x01               ;Configuration word for 6x Clk, 0 phase
     movwf   AD9850+4           ;Stuff in 4th Byte (Config. Word)                               
Mult10
     movlw   0xEF
     movwf   OP1+0
     movlw   0x00
     movwf   OP1+1
     clrf    OP1+2  
     clrf    OP1+3  
     movf    LCD_Dat+6,w       ;
     movwf   OP2               ; 
     goto    Go
Mult100
     movlw   0x52
     movwf   OP1+0             ;
     movlw   0x09              ;
     movwf   OP1+1
     clrf    OP1+2
     clrf    OP1+3  
     movf    LCD_Dat+5,w       ;
     addlw   0x00               ;       Add 0 Hz FOR  OFFSET    
     movwf   OP2
     goto    Go
Mult1K
     movlw   0x35
     movwf   OP1+0             ;
     movlw   0x5D
     movwf   OP1+1
     clrf    OP1+2
     clrf    OP1+3
     movf    LCD_Dat+4,w       ;
     addlw   0x00               ;       Add 0KHz FOR  OFFSET
     btfss   PortA,0x04         ;       If in USB Mode
     addlw   0x00               ;       add additional 0 KHz
     movwf   OP2
     goto    Go
Mult10K
     movlw   0x11
     movwf   OP1+0
     movlw   0xA4
     movwf   OP1+1
     movlw   0x03
     movwf   OP1+2
     clrf    OP1+3             ; 
     movf    LCD_Dat+3,w       ;
     addlw   0x00               ;       Add 0 KHz FOR  OFFSET 
     movwf   OP2
     goto    Go
Mult100K
     movlw   0xAD
     movwf   OP1+0
     movlw   0x68
     movwf   OP1+1
     movlw   0x24
     movwf   OP1+2
     clrf    OP1+3
     movf    LCD_Dat+2,w       ;
     addlw   0x07               ;       Add 700 KHz FOR  OFFSET 
     movwf   OP2               ; 
     goto    Go
Mult1M
     movlw   0xC1
     movwf   OP1+0
     movlw   0x16
     movwf   OP1+1
     movlw   0x6C
     movwf   OP1+2
     movlw   0x01
     movwf   OP1+3
     movf    LCD_Dat+1,w        ; 
     addlw   0x00               ;       Add 0 MHz of offset     
     movwf   OP2                ;
     goto    Go

Mult10M
     movlw   0x8E
     movwf   OP1+0
     movlw   0xE3
     movwf   OP1+1
     movlw   0x38
     movwf   OP1+2
     movlw   0x0E
     movwf   OP1+3
     movf    LCD_Dat+0,w       ;
     addlw   0X01              ; Add 10 MHz of offset           
     movwf   OP2               ;
     goto    Go
Go
     movlw   0x05               ;was 4. 5 bits of LCD_DAT used
				;when 10,700 Kc offset is used
     movwf   temp2      
DigitLoop
     bcf     Status, C         ;Clear Carry bit
     rrf     OP2,f             ;Rotate right (through carry)
     btfss   Status, C         ;See if Carry is set
     goto    NoCarry           ;Carry is clear, so jump
     call    Add32
NoCarry
     bcf     Status, C         ;Clear Carry bit
     rlf     OP1+0,f           ;Rotate left through carry
     rlf     OP1+1,f           ;
     rlf     OP1+2,f           ;
     rlf     OP1+3,f           ;
     decfsz  temp2,f 
     goto    DigitLoop
     incf    temp1,f
     movlw   0x01
     subwf   temp1,w
     btfsc   Status, Z         ;See if Zero is clear 
     goto    Mult100           ;Zero is set, so jump
     movlw   0x02
     subwf   temp1,w
     btfsc   Status, Z         ;See if Zero is clear
     goto    Mult1K            ;Zero is set, so jump  
     movlw   0x03
     subwf   temp1,w
     btfsc   Status, Z         ;See if Zero is clear
     goto    Mult10K           ;Zero is set, so jump
     movlw   0x04
     subwf   temp1,w
     btfsc   Status, Z         ;See if Zero is clear
     goto    Mult100K          ;Zero is set, so jump
     movlw   0x05
     subwf   temp1,w
     btfsc   Status, Z         ;See if Zero is clear
     goto    Mult1M            ;Zero is set, so jump
     movlw   0x06
     subwf   temp1,w
     btfsc   Status, Z         ;See if Zero is clear
     goto    Mult10M           ;Zero is set, so jump
     return
;
;
;**************************************************************************
;Initialize PIC controller
;
; Start here after reset or power on
; Set DDS initially to 10 MHz.
;
Start
     ;clrf    PortA             ; Clear data Port A
     clrf    PortB             ; Clear data Port B
     movlw   0xF0              ;
     tris    PortB             ; Enable port B drivers
     movlw   0x00              ;
     tris    PortA             ; Tristate Port A drivers
     movlw   0x01
     movwf   LCD_Dat+0
     movlw   0x00
     movwf   LCD_Dat+1
     movlw   0x00
     movwf   LCD_Dat+2
     movlw   0x00
     movwf   LCD_Dat+3
     movlw   0x00
     movwf   LCD_Dat+4
     movlw   0x00
     movwf   LCD_Dat+5
     movlw   0x00
     movwf   LCD_Dat+6
     clrf    count1
     clrf    count2
;
;**************************************************************************
;Power on initialization of Liquid Crystal Display.
; The LCD controller chip must be equivalent to an Hitachi 44780.
; The LCD is assumed to be a 1 x 16 display.
;
     movlw   0xF0              ; Enable RB.4-7 as inputs
     tris    PortB             ; 
     bcf     PortA,LCD_e
     bcf     PortA,LCD_rs
     bcf     PortA,LCD_rw
     call    Wait60ms          ; Wait for LCD to power up
     movlw   0x38              ; func set + enable to LCD
     movwf   Temp2
     call    Flipbit
     bsf     PortA,LCD_e
     call    Wait60ms          ; 
     bcf     PortA,LCD_e
     movlw   0x38              ; func set + enable to LCD
     movwf   Temp2
     call    Flipbit
     bsf     PortA,LCD_e       ; 
     call    Wait60ms          ;
     bcf     PortA,LCD_e
     movlw   0x38              ; func set + enable to LCD
     movwf   Temp2
     call    Flipbit
     bsf     PortA,LCD_e  ; 
     call    Wait60ms          ;
     bcf     PortA,LCD_e
     movlw   0x28              ; 4bit_mode instruction + enable
     movwf   Temp2
     call    Flipbit
     bsf     PortA,LCD_e
     call    Wait60ms
     bcf     PortA,LCD_e
     movlw   0x01              ; Clear and reset cursor
     movwf   temp2
     call    BusyCheck 
     bcf     PortA,LCD_rs       ;Send data to be displayed
     call    SendChar
     movlw   0x06              ;increment no_shift mode
     movwf   temp2
     call    BusyCheck
     bcf     PortA,LCD_rs       ;Send Instruction to display controller
     call    SendChar
     movlw   0x0C              ; Display on
     movwf   temp2
     call    BusyCheck
     bcf     PortA,LCD_rs
     call    SendChar
; 
;Send initial 10,000.00 display to LCD
;
     call    Mult
     call    Wait60ms          ; allow AD9850 clock time to start
;    call    Band_Set           ;Set the initial Band data
     call    WriteData         ; side effect reset power down bits of AD9850
     call    Wait60ms
     call    Wait60ms
     call    Wait60ms
     call    BCDDec
     call    WriteData 
     call    BCDInc
     call    Wait60ms
     call    WriteData         ; load the initial display value  
;
;Get the power on encoder value
;
     swapf    PortB,w           ; get encoder value
     movwf   temp2
     movlw   0x03
     andwf   temp2,w           ; isolate direction bits
     movwf   old               ; save initial values
;
;
;**************************************************************************   
;Check encoder
; PortB.4 shaft encoder output A
; PortB.5 shaft encoder output B
; PortB.6 shaft encoder push button switch
; Timing checks assume a 4MHz cpu clock and that the encoder 
; has 32 detents 
;
ChkEncoder
     incf    count1,f
     movlw   0x75              ;Check encoder loop takes 17 us
     subwf   count1,w
     btfss   Status, C         ;See if Carry is set
     goto    Ce                ;Carry is clear, so jump
     incf    count2,f
     clrf    count1            ;0x75 * 17us = 2ms
Ce
     swapf   PortB,w           ;Get encoder value, swap hi/lo nibble
     movwf   temp1
     movf    temp1,w
     movwf   temp2             ;Double latch the input          
     movlw   0x03
     andwf   temp2,w
     movwf   new
     xorwf   old,w
     btfsc   Status, Z         ;See if Zero is clear
     goto    ChkEncoder        ;Zero is set, so jump
     movlw   0x0E              ;Pointer set to 100KHz digit
     movwf   FSR
     btfss   temp2,2           ;If Tune Rate switch is closed (active low)
     goto    CeGo              ;                  
     movlw   0x10              ;Pointer set to 100's
     movwf   FSR
     ;movlw   0x10             ;timing constant 
     ;subwf   count2,w          ;If speed < 1.0 RPM
     ;btfsc   Status, C         ;See if Carry is clear
     goto    CeGo              ;Carry is set, so jump
     movlw   0x10              ;Pointer set to 1k
     movwf   FSR
     movlw   0x00               ;Use 00 as timing constant - will always set
     subwf   count2,w          ;If (1.0RPM < Speed < 3.0RPM)
     btfsc   Status, C         ;See if Carry is clear
     goto    CeGo              ;Carry is set , so jump
     movlw   0x10              ;Pointer set to 1K
     movwf   FSR
CeGo
     clrf    count2            ;Add time to write LCD etc to count1
     movlw   0x46              ;   (about 1.2 ms)
     movwf   count1       
     bcf     Status, C         ;Clear Carry bit
     rlf     old,f             ;Rotate left
     movf    new,w
     xorwf   old,f             ;Determine direction
     btfsc   old, DC           ;See if Digit Carry is clear
     goto    CeUp              ;Digit Carry is set, so jump
     call    BCDDec            ;Calculate new LCD display
     goto    CeWrite
CeUp
     call    BCDInc            ; calculate new LCD display
CeWrite 
     call    Mult              ;Calculate new AD9850 control word
     call    WriteData
     call    WriteData
     movf    new,w              ;checksum for shaft encoder/sb data
     movwf   old                ;Previous value of above.
     movf    new_sb,w          ;Shift switch state from new to old
     movwf   old_sb            ;
     goto    ChkEncoder        ; continue polling the encoder
;
;***********************************************************************
;       Sub Flipbit
;       Sets output bits for PortB LCD Data lines bit-by-bit, vs 
;       sending an entire byte to the port and corrupting the inputs.
;***********************************************************************
;
Flipbit;
     bcf     PortB,0
     btfsc   Temp2,4
     bsf     PortB,0
     bcf     PortB,1
     btfsc   Temp2,5
     bsf     PortB,1
     bcf     PortB,2
     btfsc   Temp2,6
     bsf     PortB,2
     bcf     PortB,3
     btfsc   Temp2,7
     bsf     PortB,3
     return
;
;
;    ID
;    ID
;    ID
;    ID
;    Fuses (CP=Off, PWRTE=Enabled, WDTE=Disabled, OSC=XT)
;    0001 
;
     END
